package com.eldos;

import SecureBlackbox.Base.SBEncoding;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElDCAsyncState;
import SecureBlackbox.Base.TElDCAsyncStateCollection;
import SecureBlackbox.Base.TElMemoryStream;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.PDF.TElPDFDocument;
import SecureBlackbox.PDF.TElPDFPublicKeySecurityHandler;
import SecureBlackbox.PDF.TSBPDFPublicKeySignatureType;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.io.InputStream;

@SuppressWarnings("serial")
public class ResultServlet extends HttpServlet {
	
	private static final String URL_ALL = "<a href=\"/dcweb/result?sessionId=%s&amp;name=all\">Show results</a>";
	private static final String URL = "<a href=\"/dcweb/result?name=%s\">download</a>";

    private Cache cache = Cache.getInstance();

	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String sessionID = req.getParameter("sessionid");
		
		if ((sessionID == null) || (req.getContentLength() == 0)) {
			resp.sendError(400);
			return;
		}
		if ( ! "application/xml".equals(req.getContentType())) {
			resp.sendError(415);
			return;
		}
		
		InputStream is = req.getInputStream();
		byte[] signature = SBUtils.streamToBuffer(is, 0);
		
		finishSigning(sessionID, signature);

        gotoResult(req, resp, "Signed file: " + String.format(URL_ALL, sessionID));
	}
	
	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String name = req.getParameter("name");
		
		if (name != null) { // return signed file
			if (name == "all")
			{
				String sessionID = req.getParameter("sessionid");
				
				if (sessionID == null) {
					gotoResult(req, resp, "Empty session");
					return;
				}

				String html = "";
				int k = 1;
				String s = sessionID + "." + Integer.toString(k); 
				while (cache.get(s) != null)
				{
					html += "<li>" + String.format(URL, s) + "</li>";
					k++;
					s = sessionID + "." + Integer.toString(k);
				}
					
				html = "The signed document(s) has been saved as<br /><ul>" + html + "</ul>";
				gotoResult(req, resp, html);
			}
			else
			{
				byte[] data = cache.get(name);
				if (data == null) {
					gotoResult(req, resp, "No signed file in cache");
					return;
				}
			
				resp.setContentType("application/pdf");
				resp.setHeader("Content-Disposition", "filename=sample-" + name + ".pdf");
			
				resp.getOutputStream().write(data);
				resp.getOutputStream().flush();
			}
			
		} else { // finish signing
			
			String sessionID = req.getParameter("sessionid");
			
			if (sessionID == null) {
				gotoResult(req, resp, "Empty session");
				return;
			}
				
			String data = req.getParameter("data");
			if (data != null) {
				byte[] signature = SBEncoding.base64DecodeArray(data);

				finishSigning(sessionID, signature);
			}

            gotoResult(req, resp, "Signed file: " + String.format(URL_ALL, sessionID));
		}
	}
	
	private void gotoResult(HttpServletRequest req, HttpServletResponse resp, String respStr) throws IOException {
		req.setAttribute("labelResult", respStr);
		try {
			req.getRequestDispatcher("/result.jsp").forward(req, resp);
		} catch (ServletException e) {
			resp.sendError(500);
		}
	}

	private void  finishSigning(String sessionID, byte[] signature) {
		TElDCAsyncStateCollection States = new TElDCAsyncStateCollection(true);
		try
		{
			TElMemoryStream input = new TElMemoryStream(signature, 0, signature.length);
			States.loadFromStream(input, SBDCXMLEnc.dcxmlEncoding());

			for (int i = 0; i < States.getCount(); i++)
				finishSigning(sessionID, i + 1, States.getStates(i));
		}
		finally
		{
			States.Free();
		}
	}
	
	private void finishSigning(String sessionID, Integer index, TElDCAsyncState state)
	{
        TElPDFDocument document = new TElPDFDocument();
        TElPDFPublicKeySecurityHandler handler = new TElPDFPublicKeySecurityHandler();
        handler.setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);

        try
        {
            byte[] docBin = cache.get(sessionID + "." + Integer.toString(index));
            TElMemoryStream result = new TElMemoryStream(docBin, 0, docBin.length);
            
            document.completeAsyncOperation(result, state, handler);
            
            cache.put(sessionID + "." + Integer.toString(index), result.getBuffer());
        }
        finally
        {
        	handler.Free();
        	document.Free();
        }
	}
}
