

import java.awt.EventQueue;
import java.net.URL;
import java.util.Date;

import javax.swing.*;
import SecureBlackbox.Base.SBUtils;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTable tableReq;
	private JTable tableLog;
	private ServerThread m_serverThread;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				stopClick();
				System.exit(0);
			}
		});
		
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");

		setResizable(false);
		setTitle("EldoS HTTP(S) Server Demo");
		setBounds(100, 100, 680, 423);
		getContentPane().setLayout(null);
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.setBounds(0, 0, 674, 21);
		getContentPane().add(menuBar);
		
		JMenu mnServer = new JMenu("Server");
		menuBar.add(mnServer);
		
		JMenuItem mntmStart = new JMenuItem("Start");
		mntmStart.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				startClick();
			}
		});
		mnServer.add(mntmStart);
		
		JMenuItem mntmStop = new JMenuItem("Stop");
		mntmStop.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				stopClick();
			}
		});
		mnServer.add(mntmStop);
		
		JSeparator separator = new JSeparator();
		mnServer.add(separator);
		
		JMenuItem mntmExit = new JMenuItem("Exit");
		mntmExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				dispose();
			}
		});
		mnServer.add(mntmExit);
		
		JMenu mnHelp = new JMenu("Help");
		menuBar.add(mnHelp);
		
		JMenuItem mntmAbout = new JMenuItem("About");
		mntmAbout.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				aboutClick();
			}
		});
		mnHelp.add(mntmAbout);
		
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.setBounds(0, 22, 674, 21);
		getContentPane().add(toolBar);
		
		JButton btnStart = new JButton("");
		btnStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startClick();
			}
		});
		btnStart.setFocusable(false);
		btnStart.setIcon(getIcon("start"));
		toolBar.add(btnStart);
		
		JButton btnStop = new JButton("");
		btnStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopClick();
			}
		});
		btnStop.setFocusable(false);
		btnStop.setIcon(getIcon("stop"));
		toolBar.add(btnStop);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(0, 43, 674, 224);
		getContentPane().add(scrollPane);
		
		tableReq = new JTable(new ReqTableModel());
		tableReq.setFillsViewportHeight(true);
		scrollPane.setViewportView(tableReq);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(0, 270, 674, 128);
		getContentPane().add(scrollPane_1);
		
		tableLog = new JTable(new LogTableModel());
		tableLog.setFillsViewportHeight(true);
		scrollPane_1.setViewportView(tableLog);
	}
	
	public void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	protected void aboutClick() {
		showMessage("SecureBlackbox HTTP(S) server demo.\n" +
				"Visit http://www.eldos.com/sbb/ for more information!");
	}

	protected void stopClick() {
		if (m_serverThread != null)
		{
			m_serverThread.Dispose();
			m_serverThread = null;
		}
	}

	protected void startClick() {
		if (m_serverThread == null)
		{
			SettingsDialog settingsForm = new SettingsDialog();
			settingsForm.setModal(true);
			settingsForm.setVisible(true);
			if (settingsForm.isOK())
			{
				m_serverThread = new ServerThread(new Logger(this), settingsForm.getPort(), settingsForm.getUseSSL(),
					settingsForm.getCertStorage(), settingsForm.getBasePath(), settingsForm.getAuthBasic(), settingsForm.getAuthDigest());
			}
		}
	}

	private Icon getIcon(String name) {
		URL resPath = getClass().getResource(name + ".ico");
		if (resPath == null) return new ImageIcon();
		return new ImageIcon(resPath);
	}

	public void AddLogEvent(String msg, boolean isError) {
		if (tableLog == null)
			showMessage(msg);
		else
		{
			LogTableModel model = (LogTableModel)tableLog.getModel();
			model.addRow(new Date(), msg, isError ? "Eror" : "Info");
			scrollToEndTable(tableLog);
		}
	}

	private static void scrollToEndTable(JTable tbl) {
		tbl.scrollRectToVisible(tbl.getCellRect(tbl.getRowCount() - 1, tbl.getColumnCount(), true));		
	}

	public void AddRequestEvent(String msg, String url) {
		ReqTableModel model = (ReqTableModel)tableReq.getModel();
		model.addRow(msg, url);
		scrollToEndTable(tableLog);			
	}
}
