import java.util.Date;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.TElFileStream;
import SecureBlackbox.Base.TElMemoryStream;
import SecureBlackbox.Base.TSBBoolean;
import SecureBlackbox.Base.TSBString;
import SecureBlackbox.Base.Thread;
import SecureBlackbox.PGP.*;

public class PanelProcessUndoProtectionDescriptor extends WizardPanelDescriptor {
    
    public static final String IDENTIFIER = "UNPROTECT_PROCESS_PANEL";
    
    TElPGPReader pgpReader;
    PanelProcessUndoProtection panel;
    TElPGPSignature[] sigs;
    TSBPGPSignatureValidity[] vals;
    
    public PanelProcessUndoProtectionDescriptor() {
        panel = new PanelProcessUndoProtection();
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(panel);
    }

    public Object getNextPanelDescriptor() {
        return FINISH;
    }
    
    public Object getBackPanelDescriptor() {
        return PanelSelectDestDescriptor.IDENTIFIER;
    }
        
    public void aboutToDisplayPanel() {
        panel.setProgressValue(0);
        panel.setProgressText("Decrypting file...");

        getWizard().setNextFinishButtonEnabled(false);
        getWizard().setBackButtonEnabled(false);
    }
    
    public void displayingPanel() {
   		
		Thread t = new Thread(){
    		public void run() {
	   			try
	   	    	{
	   				initReader();
	   				
	   				decryptFile(ProtectionProps.getSourcePath());

			        panel.setProgressText("Data was successfully decrypted");
	   				
			        getWizard().setNextFinishButtonEnabled(true);
			        getWizard().setBackButtonEnabled(true);
			    } catch (Exception e) {
			        
			        panel.setProgressValue(0);
			        showError(e);
			        
			        getWizard().setBackButtonEnabled(true);
			    }
    		}
		};
		
		t.start();
    }
    
	void showError(Exception e){
		panel.setProgressText("An Error Has Occurred. " + e.getMessage());
	}
	
	private void initReader() {
		pgpReader = new TElPGPReader();
		
        pgpReader.setMemoryConsumptionStrategy(TSBPGPMemoryConsumptionStrategy.mcsGreedy);
		
		pgpReader.setOnCreateOutputStream(new TSBPGPCreateOutputStreamEvent(outStreamCallback));
		pgpReader.setOnKeyPassphrase(new TSBPGPKeyPassphraseEvent(keyPassCallback));
		pgpReader.setOnPassphrase(new TSBPGPPassphraseEvent(passphCallback));
		pgpReader.setOnProgress(new TSBPGPProgressEvent(progressCallback));
		pgpReader.setOnSignatures(new TSBPGPSignaturesEvent(signCallback));	
	}
	
	private void decryptFile(String sourcePath) throws Exception {
		TElPGPKeyring keyring = new TElPGPKeyring();
		keyring.load(ProtectionProps.getPubKeyringPath(), ProtectionProps.getSecKeyringPath(), true);
		
		if (keyring.getSecretCount() == 0)
			throw new Exception("Your keyring does not contain private keys. You will not be able to decrypt encrypted files.");
		
		pgpReader.setDecryptingKeys(keyring);
		pgpReader.setVerifyingKeys(keyring);

		TElFileStream inF = new TElFileStream(sourcePath, "rw", true);
		pgpReader.decryptAndVerify(inF, 0);
		inF.Free();
	}
	
	// event's
	TSBPGPProgressEvent.Callback progressCallback = new TSBPGPProgressEvent.Callback() {
		public void tsbpgpProgressEventCallback(TObject sender, long passed, long total, TSBBoolean cancel) {
			if (total != 0) {
				int progress = (int) (passed * 100 / total); 
				panel.setProgressValue(progress);
			}
			cancel.value = false;
		}
	}; 
	
	TSBPGPSignaturesEvent.Callback signCallback = new TSBPGPSignaturesEvent.Callback() {
		public void tsbpgpSignaturesEventCallback(TObject sender, TElPGPSignature[] signatures, TSBPGPSignatureValidity[] validities) {
			TElPGPSignature sig;
			sigs = new TElPGPSignature[signatures.length];
			vals = new TSBPGPSignatureValidity[signatures.length];
			for(int i = 0; i < signatures.length; i++) 
			{
				sig = new TElPGPSignature();
				sig.assign(signatures[i]);
				sigs[i] = sig;
				vals[i] = validities[i];
			}			
		}
	};
	
	TSBPGPPassphraseEvent.Callback passphCallback = new TSBPGPPassphraseEvent.Callback() {
		
		public void tsbpgpPassphraseEventCallback(TObject sender, TSBString passphrase, TSBBoolean cancel) {
			String p = requestKeyPassphrase();
			if (p.length() == 0) {
				cancel.value = true;
			} else {
			passphrase.value = p;
			cancel.value = false;
			}
		}
		
		private String requestKeyPassphrase() {
			JPasswordField jpf = new JPasswordField();
			int result = JOptionPane.showConfirmDialog(null, jpf, "Passphrase is needed to decrypt the message", JOptionPane.OK_CANCEL_OPTION);
			if (result == 0)
				return new String(jpf.getPassword());
			return "";
		}
	};
		
	TSBPGPCreateOutputStreamEvent.Callback outStreamCallback = new TSBPGPCreateOutputStreamEvent.Callback() {
		public void tsbpgpCreateOutputStreamEventCallback(TObject sender, String fileName, Date timeStamp, TElPGPCreateStreamEventParams streamParams) {
			String target = getFileName(fileName);
			if (target.length() == 0)
				streamParams.Stream = new TElMemoryStream();
			else
				streamParams.Stream = new TElFileStream(target, "rw", true);

			streamParams.FreeOnFinish = true;
		}
		
		String getFileName(String currentFileName){
			JFileChooser fc = new JFileChooser(currentFileName);
		    int returnVal = fc.showSaveDialog(panel);
			if (returnVal == JFileChooser.APPROVE_OPTION)
		        return fc.getSelectedFile().getPath();

		    return "";
		}
	};  
	
    TSBPGPKeyPassphraseEvent.Callback keyPassCallback = new TSBPGPKeyPassphraseEvent.Callback() {
		public void tsbpgpKeyPassphraseEventCallback(TObject sender, TElPGPCustomSecretKey key, TSBString pass, TSBBoolean cancel) {
			String p = requestKeyPassphrase();
			if (p.length() == 0) {
				cancel.value = true;
			} else{
			  pass.value = p;
			  cancel.value = false;
			}
		}

		private String requestKeyPassphrase() {
			JPasswordField jpf = new JPasswordField();
			int result = JOptionPane.showConfirmDialog(null, jpf, "Passphrase is needed for secret key", JOptionPane.OK_CANCEL_OPTION);
			if (result == 0)
				return new String(jpf.getPassword());
			return "";
		}
	};
}
