import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.SSHClient.*;
import SecureBlackbox.SSHCommon.*;

class SshSession implements ForwardEventListener
{
	private TElSocket		m_clientTElSocket;
	private TElSocket		m_serverTElSocket;
	private String			m_sshHost;
	private int				m_sshPort;
	private int				m_forwardPort;
	private String			m_forwardToHost;
	private int				m_forwardToPort;
	private String			m_username;
	private String			m_password;
	private TElSSHClient	m_sshClient;
	private TElRemotePortForwardSSHTunnel m_tunnel;
	private TElSSHTunnelList m_tunnelList;
	private java.lang.Thread			m_clientThread;
	private boolean			m_error;
	private Object			m_GUILock = new Object();
	private Object			m_sshClientLock = new Object();
	ConnectionEventListener m_listener;
	
	public SshSession(ConnectionEventListener l) 
	{
		m_listener = l;
		m_clientTElSocket = new TElSocket();
		/*try {
			m_clientTElSocket.getNativeSocket().getChannel().configureBlocking(true);
		} catch (IOException e) {
			e.printStackTrace();
		}*/
		SetupComponents();
	}

	public void Connect()
	{
		Log("Connecting to " + m_sshHost + "...", false);
		m_clientThread = new java.lang.Thread(new Runnable() {
			
			public void run() {
				ClientThreadProc();
				
			}
		});
		
		m_clientThread.start();
	}

	public void Disconnect()
	{
        m_error = true;
	}

	private void ClientThreadProc() 
	{
		// establishing TCP connection
		Log("Resolving host " + m_sshHost + "...", false);
		m_error = false;
		Log("Connecting to " + m_sshHost + "...", false);
		m_clientTElSocket.setAddress(m_sshHost);
		m_clientTElSocket.setPort(m_sshPort);
		m_clientTElSocket.connect(10000);
		Log("Connected", false);
		// establishing SSH connection
		Log("Establishing SSH connection...", false);
		SetupSSHClient();
		m_sshClient.open();
		while ((!m_error) && (!m_sshClient.getActive()) && (m_clientTElSocket.getState() == SBSocket.issConnected)) 
		{
			synchronized (m_sshClientLock){
				try 
				{
					m_sshClient.dataAvailable();
				} 
				finally 
				{
				}
			}
		}

		while ((!m_error) && (m_clientTElSocket.getState() == SBSocket.issConnected))
		{
			synchronized (m_sshClientLock)
			{
				try
				{
					m_sshClient.dataAvailable();
				}
				finally
				{
				}
			}
		}

		Log("Finalizing...", false);
		if (m_sshClient.getActive()) 
		{
			m_sshClient.close(true);
		}
		if (m_clientTElSocket.getState() == SBSocket.issConnected) 
		{
			m_clientTElSocket.Free();
		}
		if (m_serverTElSocket != null) {
			m_serverTElSocket.Free();
		}
		Log("Connection shutdown", false);
	}

	private void SetupSSHClient()
	{
		m_sshClient.setUserName(m_username);
		m_sshClient.setPassword(m_password);
        m_sshClient.setAuthenticationTypes(SBSSHConstants.SSH_AUTH_TYPE_PASSWORD | SBSSHConstants.SSH_AUTH_TYPE_KEYBOARD);
		m_tunnel.setPort(m_forwardPort);
		m_tunnel.setToHost(m_forwardToHost);
		m_tunnel.setToPort(m_forwardToPort);
	}

	// Public properties

	public TElSSHClient getSshClient() 
	{
		return m_sshClient;
	}

	public String getSshHost()
	{
		return m_sshHost;
	}

	public void setSshHost(String value)
	{ 
		m_sshHost = value; 
	}

	public int getSshPort()
	{
		return m_sshPort;
	}

	public void setSshPort(int value)
	{
		 m_sshPort = value; 	
	}	
	
	public int getForwardPort()
	{
		return m_forwardPort;
	}

	public void setForwardPort(int value) 
	{ 
		m_forwardPort = value; 
	}

	public String getForwardToHost() 
	{
		return m_forwardToHost;
	}
	
	public void setForwardToHost(String value) 
	{ 
		m_forwardToHost = value; 
	}
	
	public int getForwardToPort() 
	{
		return m_forwardToPort;
	}

	public void setForwardToPort(int value) { 
		m_forwardToPort = value; 
	}

	public String getUsername()
	{
		 return m_username;
	}

	public void setUsername(String value) { 
		m_username = value; 
	}

	public String getPassword()
	{
		 return m_password;
	}

	public void setPassword(String value) { 
		m_password = value; 
	}
	

	private void SetupComponents()
	{
		// setting up tunnels
        m_tunnel = new TElRemotePortForwardSSHTunnel();
		m_tunnel.setOnClose(new TTunnelEvent(m_tunnel_OnClose));
		m_tunnel.setOnError(new TTunnelErrorEvent(m_tunnel_OnError));
		m_tunnel.setOnOpen(new TTunnelEvent(m_tunnel_OnOpen));
		m_tunnelList = new TElSSHTunnelList();
		m_tunnel.setTunnelList(m_tunnelList);

		// setting up SSH client
		m_sshClient = new TElSSHClient();
		m_sshClient.setOnAuthenticationFailed(new TSSHAuthenticationFailedEvent(m_sshClient_OnAuthenticationFailed));
		m_sshClient.setOnAuthenticationSuccess(new TNotifyEvent(m_sshClient_OnAuthenticationSuccess));
		m_sshClient.setOnCloseConnection(new TSSHCloseConnectionEvent(m_sshClient_OnCloseConnection));
		m_sshClient.setOnError(new TSSHErrorEvent(m_sshClient_OnError));
		m_sshClient.setOnKeyValidate(new TSSHKeyValidateEvent(m_sshClient_OnKeyValidate));
		m_sshClient.setOnOpenConnection(new TSSHOpenConnectionEvent(m_sshClient_OnOpenConnection));
		m_sshClient.setOnReceive(new TSSHReceiveEvent(m_sshClient_OnReceive));
		m_sshClient.setOnSend(new TSSHSendEvent(m_sshClient_OnSend));
		m_sshClient.setTunnelList(m_tunnelList);
	}

	// m_sshClient event handlers
	
	TSSHSendEvent.Callback m_sshClient_OnSend = new TSSHSendEvent.Callback() {
		
		public void tsshSendEventCallback(TObject sender, byte[] buffer) {
			if (m_clientTElSocket.getState() == SBSocket.issConnected) 
			{
				m_clientTElSocket.send(buffer, 0, buffer.length, new TSBInteger());
			} 
		}
	};
	
	TSSHReceiveEvent.Callback m_sshClient_OnReceive = new TSSHReceiveEvent.Callback() {
		
		public void tsshReceiveEventCallback(TObject sender, byte[] Buffer, int MaxSize, TSBInteger Written) {
			if (!(m_clientTElSocket.getState() == SBSocket.issConnected)) 
			{
				Written.value = 0;
			} else {
			ArrayList TElSocketList = new ArrayList();
			TElSocketList.add(m_clientTElSocket);
			if (TElSocketList.getCount() > 0)
			{
				if (m_clientTElSocket.canReceive(1000))
				{
					m_clientTElSocket.receive(Buffer, MaxSize, Written);
					if (Written.value == 0) 
					{
		                m_error = true;
					}
				}
			} 
			else 
			{
				try {
					java.lang.Thread.sleep(0);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				Written.value = 0;
			}
			}
		}
			
	};

	TSSHOpenConnectionEvent.Callback m_sshClient_OnOpenConnection = new TSSHOpenConnectionEvent.Callback() {
		
		public void tsshOpenConnectionEventCallback(TObject sender) {
	        Log("SSH connection established", false);
		}
	};
	
	TSSHAuthenticationFailedEvent.Callback m_sshClient_OnAuthenticationFailed = new TSSHAuthenticationFailedEvent.Callback() {
		
		public void tsshAuthenticationFailedEventCallback(TObject sender, int AuthenticationType) {
	        Log("Authentication " + AuthenticationType + " failed", true);
		}
	};
	
	TNotifyEvent.Callback m_sshClient_OnAuthenticationSuccess = new TNotifyEvent.Callback() {
		
		public void tNotifyEventCallback(TObject sender) {
	        Log("Authentication succeeded", false);
		}
	};
	
	TSSHCloseConnectionEvent.Callback m_sshClient_OnCloseConnection = new TSSHCloseConnectionEvent.Callback() {
		
		public void tsshCloseConnectionEventCallback(TObject sender) {
	        Log("SSH connection closed", false);
			m_error = true;		
		}
	};
	
	TSSHErrorEvent.Callback m_sshClient_OnError = new TSSHErrorEvent.Callback() {
		
		public void tsshErrorEventCallback(TObject sender, int errorCode) {
			Log("SSH protocol error " + errorCode, true);
			
			Log("If you have ensured that all connection parameters are correct and you still can't connect,", true);
			Log("please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php", true);
			Log("Remember to provide details about the error that happened.", true);
			if (m_sshClient.getServerSoftwareName().length() > 0)
			{
				Log("Server software identified itself as: " + m_sshClient.getServerSoftwareName(), true);
			}
			m_error = true;
		}
	};
	
	TSSHKeyValidateEvent.Callback m_sshClient_OnKeyValidate = new TSSHKeyValidateEvent.Callback() {
		
		public void tsshKeyValidateEventCallback(TObject sender, TElSSHKey key, TSBBoolean accept) {
			Log("Server key received", false);
			accept.value = true;
		}
	};
	
	// tunnel events
	
	TTunnelEvent.Callback m_tunnel_OnOpen = new TTunnelEvent.Callback() {
		
		public void tTunnelEventCallback(TObject sender, TElSSHTunnelConnection con) {		
			SshForwarding fwd = new SshForwarding(con, m_forwardToHost, m_forwardToPort, (SshSession)(Object)sender);
			ConnectionAdd(fwd);
			Log("Secure channel opened", false);

		}
	};
	
	TTunnelEvent.Callback m_tunnel_OnClose = new TTunnelEvent.Callback() {
		
		public void tTunnelEventCallback(TObject sender, TElSSHTunnelConnection tunnelConnection) {
	        Log("Secure channel closed", false);
			SshForwarding fwd = (SshForwarding)tunnelConnection.getData();
			if (fwd != null) 
				fwd.Close();		
		}
	};
	
	TTunnelErrorEvent.Callback m_tunnel_OnError = new TTunnelErrorEvent.Callback() {
		
		public void tTunnelErrorEventCallback(TObject sender, int error, TObject data) {
			Log("Failed to open secure channel, error " + error, true);
			SshForwarding fwd = (SshForwarding)data;
			if (fwd != null) 
				fwd.Close();		
		}
	};

	public void OnFinish(Object sender) {
		ConnectionRemove((SshForwarding)sender);
	}

	public void OnChange(Object sender) {
        ConnectionChange((SshForwarding)sender);
	}

	public void OnLog(Object sender, String s, boolean error) {
		Log(s, error);
	}

	public void OnDataSend(Object sender, byte[] buffer, int offset, int len) {
		synchronized (m_sshClientLock)
		{
			try 
			{
				SshForwarding f = (SshForwarding)sender;
				f.getSshConnection().sendData(buffer, offset, len);
			} 
			finally 
			{
			}
		}		
	}

	public void OnClose(Object sender) {
		synchronized (m_sshClientLock)
		{
			try 
			{
				SshForwarding f = (SshForwarding)sender;
				if (f.getInState() == SshForwarding.InState.Active) 
					f.getSshConnection().close(true);
			} 
			finally 
			{
			}
		}
	}
	
	private void Log(String s, boolean error) 
	{
		synchronized(m_GUILock) {
			try 
			{
				m_listener.OnLog(this, s, error);
			} 
			finally 
			{
			}
		}
	}
	
	private void ConnectionAdd(SshForwarding conn)
	{
		synchronized(m_GUILock) {
			try 
			{

				m_listener.OnConnectionAdd(this, conn);
			} 
			finally 
			{
			}
		}
	}
	
	private void ConnectionRemove(SshForwarding conn)
	{
		synchronized(m_GUILock) {
			try 
			{
				m_listener.OnConnectionRemove(this, conn);

			} 
			finally 
			{
				
			}
		}
	}

	private void ConnectionChange(SshForwarding conn)
	{
		synchronized(m_GUILock) {
			try 
			{
				m_listener.OnConnectionChange(this, conn);
			}
			finally 
			{
			}
		}
	}
		
}