import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.SFTPCommon.*;
import SecureBlackbox.SFTPServer.*;
import SecureBlackbox.SSHCommon.*;
import SecureBlackbox.SSHServer.*;

public class SFTPSession {

	private TElSSHSubsystemThread m_thread;
	private String[] m_dirList = null;
	private String[] m_fileList = null;
	private int m_dirIndex = 0;
	private int m_fileIndex = 0;

	public SFTPSession(TElSSHTunnelConnection conn) {
		m_thread = new TElSSHSubsystemThread(new TElSFTPSSHSubsystemHandler(conn, true), conn, true);
		
		initEvents();

		m_thread.resume();
	}

	private void initEvents() {
		TElSFTPSSHSubsystemHandler handler = (TElSFTPSSHSubsystemHandler)m_thread.getHandler();
		TElSFTPServer s = handler.getServer();
		
		s.setOnClose(new TNotifyEvent(closeEvent));
		s.setOnCloseHandle(new TElSFTPServerCloseHandleEvent(closeHandleEvent));
		s.setOnCreateDirectory(new TElSFTPServerCreateDirectoryEvent(createDirEvent));
		s.setOnError(new TSBSftpErrorEvent(errorEvent));
		s.setOnFindClose(new TElSFTPServerFindCloseEvent(findCloseEvent));
		s.setOnFindFirst(new TElSFTPServerFindFirstEvent(findFirstEvent));
		s.setOnFindNext(new TElSFTPServerFindNextEvent(findNextEvent));
		s.setOnOpen(new TNotifyEvent(openEvent));
		s.setOnOpenFile(new TElSFTPServerOpenFileEvent(openFileEvent));
		s.setOnReadFile(new TElSFTPServerReadEvent(readFileEvent));
		s.setOnRemove(new TElSFTPServerRemoveEvent(removeEvent));
		s.setOnRenameFile(new TElSFTPServerRenameFileEvent(renameFileEvent));
		s.setOnRequestAbsolutePath(new TElSFTPServerRequestAbsolutePathEvent(reqAbsolutePathEvent));
		s.setOnRequestAttributes(new TElSFTPServerRequestAttributesEvent(reqAttrEvent));
		s.setOnRequestAttributes2(new TElSFTPServerRequestAttributes2Event(reqAttr2Event));
		s.setOnWriteFile(new TElSFTPServerWriteEvent(writeFileEvent));		
		s.setVersions((short) (SBSftpCommon.sbSFTP3 | SBSftpCommon.sbSFTP4 | SBSftpCommon.sbSFTP5 | SBSftpCommon.sbSFTP6));
		
		Logger.Log("SFTP server started");
	}
	
	TElSFTPServerWriteEvent.Callback writeFileEvent = new TElSFTPServerWriteEvent.Callback() {
		public void tElSFTPServerWriteEventCallback(TObject sender, TObject data,
				long offset, byte[] buffer, int bufferOffset, int count, TSBInteger errCode,
				TSBString comment) {
			try 
			{
				SFTPFileStream strm = (SFTPFileStream)(Object)data;
				strm.setPosition(offset);
				strm.write(buffer, bufferOffset, count);
				errCode.value = 0;
			} 
			catch(Exception ex) 
			{
				errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;
				comment.value = ex.getMessage();
			}
			
		}
	};
	
	TElSFTPServerRequestAttributes2Event.Callback reqAttr2Event = new TElSFTPServerRequestAttributes2Event.Callback() {
		
		public void tElSFTPServerRequestAttributes2EventCallback(TObject sender,
				TObject data, TElSftpFileAttributes attrs, TSBInteger errCode,
				TSBString comment) {
			if ((Object)data instanceof SFTPFileStream) 
			{
				try 
				{
					SFTPFileStream f = (SFTPFileStream)(Object)data;
					String path = f.getPath();
					GetAttributes(path, attrs);
					errCode.value = 0;
				} 
				catch(Exception ex) 
				{
                    errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;
					comment.value = ex.getMessage();
				}
			}
		}
	};
	
	TElSFTPServerRequestAttributesEvent.Callback reqAttrEvent = new TElSFTPServerRequestAttributesEvent.Callback() {

		public void tElSFTPServerRequestAttributesEventCallback(TObject sender,
				String path, boolean followSymLinks, TElSftpFileAttributes attrs,
				TSBInteger errCode, TSBString comment) {
            String localpath = CanonicalizePath(path, true);
			try 
			{
				if (new File(localpath).exists()) 
				{
					GetAttributes(localpath, attrs);
					errCode.value = 0;
				}
				else 
				{
                    errCode.value = SBSftpCommon.SSH_ERROR_NO_SUCH_FILE;
					comment.value = "No such file";
				}
			} 
			catch(Exception ex) 
			{
                errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;
				comment.value = ex.getMessage();
			}			
		}
	};
	
	TElSFTPServerRequestAbsolutePathEvent.Callback reqAbsolutePathEvent = new TElSFTPServerRequestAbsolutePathEvent.Callback() {
		public void tElSFTPServerRequestAbsolutePathEventCallback(TObject sender,
				String path, TSBString absolutePath, TSBSftpRealpathControl control,
				TElStringList composePath, TSBInteger errCode, TSBString comment) {
			try
			{
				absolutePath.value = CanonicalizePath(path, false);
				errCode.value = 0;
			} catch(Exception ex){
				ex.printStackTrace();
				Logger.Log(ex.getMessage());
			}
		}
	};
	
	TElSFTPServerRenameFileEvent.Callback renameFileEvent = new TElSFTPServerRenameFileEvent.Callback() {

		public void tElSFTPServerRenameFileEventCallback(TObject sender, String oldPath,
				String newPath, int flags, TSBInteger errCode, TSBString comment) {
			boolean success = false;
			String localold = CanonicalizePath(oldPath, true);
			String localnew = CanonicalizePath(newPath, true);
			try 
			{
				File oldFile = new File(localold);
				File newFile = new File(localnew);

				success = oldFile.renameTo(newFile);
				
				if (!success) 
				{
                    throw new Exception("File does not exist");
				}
				errCode.value = 0;
			} 
			catch(Exception ex) 
			{
                errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;
				comment.value = ex.getMessage();
			}
		}
	};
	
	TElSFTPServerRemoveEvent.Callback removeEvent = new TElSFTPServerRemoveEvent.Callback() {
		
		public void tElSFTPServerRemoveEventCallback(TObject sender, String path,
				TSBInteger errCode, TSBString comment) {
			boolean success = false;
            String localpath = CanonicalizePath(path, true);
			try 
			{
				File item = new File(localpath);
			    // Make sure the file or directory exists and isn't write protected
			    if (!item.exists())
			      throw new IllegalArgumentException(
			          "Delete: no such file or directory: " + item.getName());

			    if (!item.canWrite())
			      throw new IllegalArgumentException("Delete: write protected: "
			          + item.getName());
			    
				success = item.delete();
				
				if (!success) 
				{
                    throw new Exception("Can not delete file");
				} 
				errCode.value = 0;
			} 
			catch(Exception ex) 
			{
                errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;           
				comment.value = ex.getMessage();
			}
		}
	};
	
	TElSFTPServerReadEvent.Callback readFileEvent = new TElSFTPServerReadEvent.Callback() {

		public void tElSFTPServerReadEventCallback(TObject sender, TObject data,
				long offset, byte[] buffer, int bufferOffset, int count, TSBInteger read,
				TSBInteger errCode, TSBString comment) {
			SFTPFileStream strm = (SFTPFileStream)(Object)data;
			try {
				strm.setPosition(offset);
			} catch (IOException e) {
				e.printStackTrace();
				Logger.Log(e.getMessage());
			}
			read.value = strm.read(buffer, bufferOffset, count);
			if (read.value > 0) 
			{
				errCode.value = 0;
			} 
			else 
			{
				errCode.value = SBSftpCommon.SSH_ERROR_EOF;
			}
		}
	};
	
	TElSFTPServerOpenFileEvent.Callback openFileEvent = new TElSFTPServerOpenFileEvent.Callback() {
		public void tElSFTPServerOpenFileEventCallback(TObject sender, String path,
				int modes, int access, int desiredAccess, TElSftpFileAttributes attrs,
				TSBObject data, TSBInteger errCode, TSBString comment) {
			String realpath = CanonicalizePath(path, true);
			try 
			{
				SFTPFileStream stream = new SFTPFileStream(realpath, modes);
				if ((modes & SBSftpCommon.fmAppend) == SBSftpCommon.fmAppend) 
				{
					long len = stream.getLength();
					stream.setPosition(len);
				}
				data.value = stream;
				errCode.value = 0;
			} 
			catch(Exception ex) 
			{
				errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;
				comment.value = ex.getMessage();
			}
		}
	};
	
	TNotifyEvent.Callback openEvent = new TNotifyEvent.Callback() {
		
		public void tNotifyEventCallback(TObject sender) {
			Logger.Log("SFTP connection established");
		}
	};
	
	TElSFTPServerFindNextEvent.Callback findNextEvent = new TElSFTPServerFindNextEvent.Callback() {
		
		public void tElSFTPServerFindNextEventCallback(TObject sender, TObject data,
				TElSftpFileInfo info, TSBInteger errCode, TSBString comment) {
			try
			{
				errCode.value = FillNextFileInfo(info, errCode.value);
			} catch(Exception ex){
				ex.printStackTrace();
				Logger.Log(ex.getMessage());
			}
		}
	};
	
	TElSFTPServerFindFirstEvent.Callback findFirstEvent = new TElSFTPServerFindFirstEvent.Callback() {
		
		public void tElSFTPServerFindFirstEventCallback(TObject sender, String path,
				TSBObject data, TElSftpFileInfo info, TSBInteger errCode,
				TSBString comment) {
			String localpath = CanonicalizePath(path, true);
			try 
			{ 
				String[] lst = GetDirectories(localpath);
				m_dirList = new String[lst.length + 2];
				m_dirList[0] = ".";
				m_dirList[1] = "..";
				for (int i = 0; i < lst.length; i++) 
				{
					m_dirList[i + 2] = lst[i];
				}
				m_fileList = GetFiles(localpath);
				m_fileIndex = 0;
				m_dirIndex = 0;
				errCode.value = FillNextFileInfo(info, errCode.value);
				data = null;
			} 
			catch(Exception ex) 
			{
                errCode.value = SBSftpCommon.SSH_ERROR_NO_SUCH_FILE;
				comment.value = ex.getMessage();
			}
		}
	};
		
	TElSFTPServerFindCloseEvent.Callback findCloseEvent = new TElSFTPServerFindCloseEvent.Callback() {
		
		public void tElSFTPServerFindCloseEventCallback(TObject sender, TObject searchRec,
				TSBInteger errCode, TSBString comment) {
            errCode.value = 0;
		}
	};
	
	TSBSftpErrorEvent.Callback errorEvent = new TSBSftpErrorEvent.Callback() {
		
		public void tsbSftpErrorEventCallback(TObject sender, int errCode, String comment) {
            Logger.Log("SFTP client error: " + errCode, true);
		}
	};
	
	TElSFTPServerCreateDirectoryEvent.Callback createDirEvent = new TElSFTPServerCreateDirectoryEvent.Callback() {
		
		public void tElSFTPServerCreateDirectoryEventCallback(TObject sender,
				String path, TElSftpFileAttributes attrs, TSBInteger errCode,
				TSBString comment) {
            String realpath = CanonicalizePath(path, true);
			try 
			{
				new File(realpath).mkdir();
				errCode.value = 0;
			} 
			catch(Exception ex) 
			{
				errCode.value = SBSftpCommon.SSH_ERROR_PERMISSION_DENIED;
				comment.value = ex.getMessage();
			}
		}
	};

	TElSFTPServerCloseHandleEvent.Callback closeHandleEvent = new TElSFTPServerCloseHandleEvent.Callback() {
		public void tElSFTPServerCloseHandleEventCallback(TObject sender,
				TObject data, TSBInteger errCode, TSBString comment) {
			try 
			{
				((SFTPFileStream)(Object)data).close();
				errCode.value = 0;
			} 
			catch(Exception ex) 
			{
                errCode.value = SBSftpCommon.SSH_ERROR_FAILURE;
				comment.value = ex.getMessage();
			}
		}
	};
	
	TNotifyEvent.Callback closeEvent = new TNotifyEvent.Callback() {
		
		public void tNotifyEventCallback(TObject sender) {
			Logger.Log("SFTP connection closed");
		}
	};

	// methods
	private void GetAttributes(String path, TElSftpFileAttributes attrs) throws Exception
	{
		File item = new File(path);
		if (!item.exists())
			throw new Exception("File does not exist");
		
		if (item.isDirectory()) 
		{
			attrs.setSize(0);
			attrs.setDirectory(true);
			attrs.setFileType(TSBSftpFileType.ftDirectory);
			attrs.setMTime(new Date(item.lastModified()));
			attrs.setATime(new Date(item.lastModified()));
			attrs.setCTime(new Date(item.lastModified()));

			attrs.setUserWrite(true);
			attrs.setGroupWrite(true);
			attrs.setOtherWrite(true);
			

		} 
		else if (item.isFile()) 
		{
			attrs.setSize(item.length());
			attrs.setDirectory(false);
			attrs.setFileType(TSBSftpFileType.ftFile);
			attrs.setMTime(new Date(item.lastModified()));
			attrs.setATime(new Date(item.lastModified()));
			attrs.setCTime(new Date(item.lastModified()));
		} 

		attrs.setUserRead(true);
		attrs.setGroupRead(true);
		attrs.setOtherRead(true);
	
		attrs.setIncludedAttributes(SBSftpCommon.saPermissions | SBSftpCommon.saSize |
			SBSftpCommon.saMTime | SBSftpCommon.saATime | SBSftpCommon.saCTime);
	}	

	private String CanonicalizePath(String relpath, boolean local)
	{
		String[] pathelems = relpath.split("/", 1024);
		Vector<String> lst = new Vector<String>();
		for (int i = 0; i < pathelems.length; i++) 
		{
			String elem = (String)pathelems[i];
			if (elem.compareTo("..") == 0) 
			{
				if (lst.size() > 0) 
				{
					lst.remove(lst.size() - 1);
				}
			} 
			else if ((elem.compareTo(".") != 0) && (elem.compareTo("") != 0))
			{
				lst.add(elem);
			}
		}
		String result;
		String separator;
		if (local) 
		{
			result = "C:\\";
			separator = "\\";
		} 
		else 
		{
			result = "/";
			separator = "/";
		}
		for (int i = 0; i < lst.size(); i++) 
		{
			result = result + (String)lst.get(i);
			if (i != lst.size() - 1) 
			{
				result = result + separator;
			}
		}
		return result;
	}
	
	String[] GetDirectories(String path) {
		Vector<String> result = new Vector<String>();
		File items[] = new File(path).listFiles();
		 for (int i = 0; i < items.length; i++) {
             if (items[i].isDirectory()) {
            	 result.add(items[i].getAbsolutePath());
             }
		 }
		 return (String[]) result.toArray(new String[result.size()]);
	}
	
	String[] GetFiles(String path) {
		Vector<String> result = new Vector<String>();
		File items[] = new File(path).listFiles();
		 for (int i = 0; i < items.length; i++) {
             if (items[i].isFile()) {
            	 result.add(items[i].getAbsolutePath());
             }
		 }
		 return (String[]) result.toArray(new String[result.size()]);
	}
	
	private int FillNextFileInfo(TElSftpFileInfo info, int ErrorCode)
	{
		TElSftpFileAttributes attrs = info.getAttributes();
		attrs.setPermissions(0);
		if (m_dirIndex < m_dirList.length) 
		{
			File dir = new File(m_dirList[m_dirIndex]);
			info.setName(dir.getName());
			attrs.setCTime(new Date(dir.lastModified()));
			attrs.setATime(new Date(dir.lastModified()));
			attrs.setMTime(new Date(dir.lastModified()));
			attrs.setSize(0);
			attrs.setDirectory(true);
			attrs.setFileType(TSBSftpFileType.ftDirectory);
			m_dirIndex++;
		} 
		else if (m_fileIndex < m_fileList.length) 
		{
			File file = new File(m_fileList[m_fileIndex]);

			info.setName(file.getName());
			attrs.setCTime(new Date(file.lastModified()));
			attrs.setATime(new Date(file.lastModified()));
			attrs.setMTime(new Date(file.lastModified()));
			attrs.setSize(file.length());
			attrs.setDirectory(false);
			attrs.setFileType(TSBSftpFileType.ftFile);
			m_fileIndex++;
		} 
		else 
		{
			ErrorCode = SBSftpCommon.SSH_ERROR_EOF;
			return ErrorCode;
		}
		attrs.setUserExecute(true);
		attrs.setUserRead(true);
		attrs.setUserWrite(true);
		attrs.setGroupExecute(true);
		attrs.setGroupRead(true);
		attrs.setGroupWrite(true);
		attrs.setOtherExecute(true);
		attrs.setOtherRead(true);
		attrs.setOtherWrite(true);
		attrs.setIncludedAttributes(SBSftpCommon.saPermissions | SBSftpCommon.saSize |
			SBSftpCommon.saMTime | SBSftpCommon.saATime | SBSftpCommon.saCTime);
		info.setLongName(UnixPath(info));
		
		return ErrorCode;
	}
	
	private String UnixPath(TElSftpFileInfo info) 
	{
		String result = "";
		if (info.getAttributes().getDirectory()) 
		{
			result = "drwxrwxrwx   1 user     group    ";
		} 
		else 
		{
			result = "-rwxrwxrwx   1 user     group    ";
		}
		String szstr = "" + info.getAttributes().getSize();
		while (szstr.length() < 8) 
		{
			szstr = "0" + szstr;
		}
		result = result + szstr + " ";
		result = result + " " + fmt.format(info.getAttributes().getMTime()) + " " + info.getName();
		return result;
	}
  
  private static final SimpleDateFormat fmt = new SimpleDateFormat("MMM dd HH:mm", Locale.US);
}
