import java.util.*;
import SecureBlackbox.Base.TElSocket;

public class ServerListener {

	private Object m_synch = null;
	private ArrayList<SSHSession> m_arrSessions = null; 	
	private TcpListener m_SocketListener = null;
	private ManualResetEvent m_Event = null;
	private Thread m_Thread = null;
	private int m_MaxSocketSessions = 20;
	private int m_ClientCount = 0;
	private SessionEventListener m_eListener;

	public ServerListener(SessionEventListener e)
	{
		m_eListener = e;
		m_synch = new Object();
		m_arrSessions = new ArrayList<SSHSession>();
		m_Event = new ManualResetEvent(false);
		m_Thread = new java.lang.Thread(new Runnable() {
			public void run() {
				try {
					listen();
				} catch (InterruptedException e) {
					Logger.Log(e.getMessage());
				}
			}
		}, "SSH_Server_Listener_Thread");
	}
	
	private void listen() throws InterruptedException {
		acceptSocket();			
	}

	private void acceptSocket()
	{
		try
		{
			while(m_Event.waitOne()) {
				if (m_SocketListener.pending()) 
					m_SocketListener.accept();
				else
					Thread.sleep(500);
			}
		}
		catch(Exception ex)
		{
			Logger.Log("AcceptSocket() : " + ex.getMessage(),true);
		}
	}


	public boolean start() {
		try
		{			
			Logger.Log("Starting SSH server listener...");
			String host = Globals.getSettings().getServerHost();
			int port = Globals.getSettings().getServerPort();
			m_SocketListener = new TcpListener(host, port) {
				@Override
				void started(TElSocket socket) {
					Logger.Log("SSH server listener started.");
				}

				@Override
				void connected(TElSocket socket) {
					Logger.Log("ServerListener.Listen : New connection available");
					if (socket != null) {
						synchronized(m_synch)
						{					
							if(m_arrSessions.size() < m_MaxSocketSessions)
							{
								//create new session and start it in a new thread							
								SSHSession session = new SSHSession(socket, ++m_ClientCount, m_eListener);
								m_arrSessions.add(session);
								Logger.Log("Connection accepted. Active connections "  + m_arrSessions.size() + " from " + m_MaxSocketSessions);
								session.start();
								m_eListener.SessionStarted(session);
							}
							else
							{	
								Logger.Log("New connection rejected");
								try
								{						
									socket.close(false);
								}
								catch(Exception ex)
								{
									;
								}						
							}
						}
					}
				}
			};
			m_SocketListener.start();	
			m_Thread.start();	
			m_Event.set();
						
			if (Globals.getMain() != null)
			{
				synchronized(Globals.getMain())
				{
					Globals.getMain().setTopButtonsEnabled(0, false);
					Globals.getMain().setTopButtonsEnabled(1, true);
				}
			}
			Globals.setServerStarted(true);
			return true;
		}
		catch(Exception exc)
		{
			Logger.Log("ServerListener.Start : " + exc.getMessage(), true);
			return false;
		}
	}

	private void stop() {
		try
		{
			if (m_SocketListener != null) 
			{
				Logger.Log("Stopping socket listener");	
				m_SocketListener.stop();
				if (Globals.getMain() != null)
				{
					synchronized(Globals.getMain())
					{
						Globals.getMain().setTopButtonsEnabled(0, true);
						Globals.getMain().setTopButtonsEnabled(1, false);
					}
				}
				Globals.setServerStarted(false);
				Logger.Log("Socket listener stopped");	
			}
		}
		catch(Exception ex)
		{
			Logger.Log("Exception while stopping socket listener : " + ex.getMessage());
		}
	}

	public void dispose() throws InterruptedException {
		if ((m_Thread != null) && (m_Thread.getState() != Thread.State.NEW))
		{		
			m_Event.reset();
			stop();

			//kill thread
			m_Thread.interrupt();
			m_Thread.join(1000);												
			m_Event.close();

			//kill sockets (sessions)
			synchronized(m_synch)
			{				
				for (int i = 0, N = m_arrSessions.size(); i < N; ++i)
				{//stop all sessions
					SSHSession session=(SSHSession)m_arrSessions.get(i);
					assert(session!=null);
					session.closeSession();
				}
				m_arrSessions.clear();
			}
			m_Thread = null;
		}
	}
}
