import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JPasswordField;
import javax.swing.JTable;
import javax.swing.JScrollPane;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.UUID;


public class Settings extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField txtBasePath;
	private JTextField txtPort;
	private JTextField txtFileStore;
	private JTextField txtCertFile;
	private JPasswordField passwordField;
	private JTable tbUsers;
	boolean isOk;
	private JCheckBox cbUseSSL;
	private TElDiskFileSystemAdapter m_diskAdapter;
	private TElMemoryCertStorage m_certStorage; 
	private String m_attrFilePath;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			Settings dialog = new Settings();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void init() {
		m_diskAdapter = new TElDiskFileSystemAdapter();
		
		m_diskAdapter.setOnWriteFileTag(new TElVFSWriteFileTagEvent(new TElVFSWriteFileTagEvent.Callback() {
			@Override
			public void tElVFSWriteFileTagEventCallback(TObject arg0, String Path, String TagName, String TagValue,
					TSBInteger Result) {
				Result.value = 0;
				
				if (TagValue == null || TagValue.length() == 0)
					return;
				
				try
				{
					JIniFile ini = new JIniFile(m_attrFilePath);
					ini.WriteString(URLRemoveSlash(Path), TagName, TagValue);
					ini.UpdateFile();
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
					Result.value = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
					return;
				}
			}
		}));
		
		m_diskAdapter.setOnReadFileTag(new TElVFSReadFileTagEvent(new TElVFSReadFileTagEvent.Callback() {
			@Override
			public void tElVFSReadFileTagEventCallback(TObject arg0, String Path, String TagName, TSBString TagValue,
					TSBInteger Result) {
				try
				{
					JIniFile ini = new JIniFile(m_attrFilePath);
					TagValue.value = ini.ReadString(URLRemoveSlash(Path), TagName, "");
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
					Result.value = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
					return;
				}
				Result.value = 0;
			}
		}));
		
		m_diskAdapter.setOnDeleteFileTag(new TElVFSDeleteFileTagEvent(new TElVFSDeleteFileTagEvent.Callback() {
			@Override
			public void tElVFSDeleteFileTagEventCallback(TObject arg0, String Path, String TagName, TSBInteger Result) {
				// TODO Auto-generated method stub
				try
				{
					JIniFile ini = new JIniFile(m_attrFilePath);
					ini.EraseSection(URLRemoveSlash(Path));
					ini.UpdateFile();
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
					Result.value = SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
					return;
				}

				Result.value = 0;
			}
			
			
		}));

        String basePath = CreateTempPath();
        
        String appPath = new java.io.File("").getAbsolutePath();
        String appDir = new java.io.File(appPath).getParentFile().getName();
        if ((basePath) == "")
            txtBasePath.setText(appDir);
        else
            txtBasePath.setText(basePath);

		txtCertFile.setText(appPath + "\\cert.pem");
		txtFileStore.setText(appPath + "\\attr.db");
	}

	private String getTempFileName() {
		return System.getProperty("java.io.tmpdir") + UUID.randomUUID().toString();
	}
	
	private String CreateTempPath() {
		String tempFileName = getTempFileName();
		new File(tempFileName).mkdirs();
		return tempFileName;
	}
	
	private String URLRemoveSlash(String URL)
	{
		if ((URL.length() > 0) && (! URL.equals("/")) && (URL.charAt(URL.length() - 1) == '/'))
		{
			return URL.substring(0, URL.length() - 1);
		}
		else
		{
			return URL;
		}
	}
	
	private UserTableModel model; 
	/**
	 * Create the dialog.
	 */
	public Settings() {
		setTitle("Server Settings");
		setBounds(100, 100, 523, 430);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblBasePath = new JLabel("Base path:");
		lblBasePath.setBounds(10, 11, 348, 14);
		contentPanel.add(lblBasePath);
		
		txtBasePath = new JTextField();
		txtBasePath.setBounds(10, 31, 395, 20);
		contentPanel.add(txtBasePath);
		txtBasePath.setColumns(10);
		
		JButton button = new JButton("...");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtBasePath.setText(getSelectedFolder());
			}
		});
		button.setBounds(415, 30, 35, 23);
		contentPanel.add(button);
		
		JLabel lblListenOnPort = new JLabel("Listen on port:");
		lblListenOnPort.setBounds(10, 65, 86, 14);
		contentPanel.add(lblListenOnPort);
		
		txtPort = new JTextField();
		txtPort.setText("8080");
		txtPort.setBounds(106, 62, 86, 20);
		contentPanel.add(txtPort);
		txtPort.setColumns(10);
		
		cbUseSSL = new JCheckBox("Use SSL/TLS");
		cbUseSSL.setBounds(198, 61, 132, 23);
		contentPanel.add(cbUseSSL);
		
		JLabel lblNewLabel = new JLabel("File to store attributes:");
		lblNewLabel.setBounds(10, 100, 218, 14);
		contentPanel.add(lblNewLabel);
		
		txtFileStore = new JTextField();
		txtFileStore.setColumns(10);
		txtFileStore.setBounds(10, 119, 395, 20);
		contentPanel.add(txtFileStore);
		
		JButton button_1 = new JButton("...");
		button_1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtFileStore.setText(getSourceFile());
			}
		});
		button_1.setBounds(415, 118, 35, 23);
		contentPanel.add(button_1);
		
		JLabel lblCertificateFile = new JLabel("Certificate file:");
		lblCertificateFile.setBounds(10, 150, 152, 14);
		contentPanel.add(lblCertificateFile);
		
		txtCertFile = new JTextField();
		txtCertFile.setColumns(10);
		txtCertFile.setBounds(10, 169, 395, 20);
		contentPanel.add(txtCertFile);
		
		JButton button_2 = new JButton("...");
		button_2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtCertFile.setText(getCertFile());
			}
		});
		button_2.setBounds(415, 168, 35, 23);
		contentPanel.add(button_2);
		
		JLabel lblCertificatePassword = new JLabel("Certificate password:");
		lblCertificatePassword.setBounds(10, 200, 233, 14);
		contentPanel.add(lblCertificatePassword);
		
		passwordField = new JPasswordField();
		passwordField.setBounds(10, 218, 395, 20);
		contentPanel.add(passwordField);
		
		JLabel lblAuthorisedUsers = new JLabel("Authorised users");
		lblAuthorisedUsers.setBounds(10, 249, 152, 14);
		contentPanel.add(lblAuthorisedUsers);
		
		JButton btnAdd = new JButton("Add");
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addUser();
			}
		});
		btnAdd.setBounds(415, 270, 91, 23);
		contentPanel.add(btnAdd);
		
		JButton btnRemove = new JButton("Remove");
		btnRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeUser();
			}
		});
		btnRemove.setBounds(415, 304, 91, 23);
		contentPanel.add(btnRemove);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 274, 395, 92);
		contentPanel.add(scrollPane);
		
		model = new UserTableModel();
		tbUsers = new JTable(model);
		tbUsers.setFillsViewportHeight(true);
		scrollPane.setViewportView(tbUsers);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setVisible(false);
						isOk = true;
						
						m_attrFilePath = getAttributesDBPath();
						m_diskAdapter.setBasePath(getBasePath());
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setVisible(false);
						isOk = false;
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		init();
	}

	protected void removeUser() {
		model.remove(tbUsers.getSelectedRow());
	}

	protected void addUser() {
		UserAdd form = new UserAdd();
		form.setModal(true);
		form.setVisible(true);
		if (form.isOk()) {
			String name = form.getUserName();
			String pass = form.getPassword();
			model.addRow(name, pass);
		}
	}

	protected String getCertFile() {
		JFileChooser fc = new JFileChooser();

		FileNameExtensionFilter filter = new FileNameExtensionFilter("Raw X.509 certificate", "cer", "csr", "crt", "pem", "pfx", "p12");
	    fc.setFileFilter(filter);		 
	    
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	protected String getSourceFile() {
		JFileChooser fc = new JFileChooser();
	    
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	protected String getSelectedFolder() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	public boolean isOk() {
		return isOk;
	}

	public String getAttributesDBPath() {
		return txtFileStore.getText();
	}

	public String getBasePath() {
		return txtBasePath.getText();
	}

	public TElCustomFileSystemAdapter getDiskAdapter() {
		return m_diskAdapter;
	}

	public TElCustomCertStorage getCertStorage() {
		return m_certStorage;
	}

	public TElStringList getUsers() {
        return model.getUsers();
	}

	public boolean getUseSSL() {
		return cbUseSSL.isSelected();
	}

	public int getPort() {
		return Integer.parseInt(txtPort.getText());
	}
}
